@@ -60,6 +60,7 @@ bint getfreespace(char *path);
 int findmax(char *subject, int *this);
 int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint maxlines);
 int dl_servers(char *serverpat, char *grouppat, int options, int wait_after, int wait_secs, char *tmpdir);
+int findfilename(regex_t *preq, char *subject, char *fnamebuf, int *curpart, int *maxparts);
 
 int comp(struct newsmsg *a, struct newsmsg *b)
 {
@@ -651,18 +652,15 @@ bint getfreespace(char *path)
 int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint maxlines)
 {
 	regex_t preq;
-	regmatch_t pmatch[1];
 	char fnamebuf[256];
-	int gotsum=0;
-	int laststart=0,lastend=0;
-	int i=0,j=0,lincnt;
+	int i=0,lincnt;
 	int howmany=0;
 	int maxparts=0;
+	int curpart;
 	int accepted_msgs=0;
 	bint acceptedlines=0;
 	FILE *dp;
 	FILE *qp=NULL;
-	char *tmp, *oldtmp=NULL;
 
 	dp=fopen("suckothermsgs", "w");
 	if(!dp)
@@ -682,6 +680,8 @@ int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint
 		}
 	}
 
+	regcomp(&preq, Filetypes, REG_ICASE|REG_EXTENDED);
+
 	if((options & OPT_LENIENT) || (options & OPT_SLOPPY))
 	{
 		printf("[PIMPPA] Using %s mode, taking all articles...\n",
@@ -691,63 +691,42 @@ int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint
 		{
 			acceptedlines+=atoi(&linebuf[i].lines[3]);
 			fprintf(dp, "%s\n", &linebuf[i].id[3]); 
+
+			if(options & OPT_INSERTSUBJECTS) {
+		        if(findfilename(&preq,linebuf[i].subject,
+								fnamebuf,&curpart,&maxparts)==RESULT_OK) {
+  	  	  	  		fprintf(qp, "%s%c%s%c", fnamebuf, 0,
+								linebuf[i].subject, 0);
+				}
+			}
 		}
 		if(acceptedlines>maxlines)
 			printf("[PIMPPA] HD shortage, not downloading all articles (L)");
 		
 		fclose(dp);
+		if(options & OPT_INSERTSUBJECTS)
+		  fclose(qp);
+	    regfree(&preq);
 		return(messages);
 	}
 
-	regcomp(&preq, Filetypes, REG_ICASE|REG_EXTENDED);
-
 	lincnt=0;
 	while(lincnt<messages)
 	{
-		char * subject;
-		
+	    char *subject;
+			
 		howmany=1;
 		subject=linebuf[lincnt].subject;
-		tmp=subject;
 		if(Verbose)
-			printf("%s\n", tmp);
-		i=0;
-		oldtmp=NULL;
-		// Find out the last matching filename
-		while(regexec(&preq, tmp, 1, pmatch, 0)!=REG_NOMATCH)
-		{
-			laststart=pmatch[0].rm_so;
-			lastend=pmatch[0].rm_eo;
-			oldtmp=tmp;
-			tmp=&tmp[lastend];
-			gotsum=1;
-		}
-		
-		if(oldtmp)
-		{
+			printf("%s\n", subject);
+
+		if(findfilename(&preq,subject,fnamebuf,&curpart,&maxparts)==RESULT_OK) {
 			int chkval;
 			int subjval;
-			int curpart;
-
-			tmp=oldtmp;
-			
-			j=0;
-			for(i=laststart;
-				i>=0 && tmp[i]!=' ' && tmp[i]!='\"' && tmp[i]!='/' && tmp[i]!='\\';
-				i--);
-			
-			i++;
-			//if(tmp[i]==' ') i++;
-			
-			while(i<lastend)
-				fnamebuf[j++]=tmp[i++];
-			fnamebuf[j]=0;
 
 //			if(Verbose)
 //				printf("  FILE %s\n", fnamebuf);
 
-			maxparts=findmax(&tmp[i], &curpart);
-
 			if(curpart==0 && maxparts>0)
 				howmany=0;
 
@@ -760,13 +739,13 @@ int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint
 
 			subjval=p_checksubject(src_db, destarea,subject);
 			if(subjval<0) {
-			  if(Verbose)
-  	 	  	    printf("  SKIP %s: Rejected keyword found in the subject\n",
-				  fnamebuf);
-			  continue;
+				if(Verbose)
+  	 	  	       printf("  SKIP %s: Rejected keyword found in the subject\n",
+				   fnamebuf);
+ 	 	 	   continue;
 			} 
 
-                        if(options & OPT_KEYNAZI)
+            if(options & OPT_KEYNAZI)
   	  		  chkval=p_checkfile(src_db, destarea, fnamebuf, (options | OPT_NAZI));
   	  		else
 			  chkval=p_checkfile(src_db, destarea, fnamebuf, options);
@@ -812,9 +791,7 @@ int parse(int destarea, struct newsmsg *linebuf, int messages, int options, bint
 					accepted_msgs++;
 				}
 				if(options & OPT_INSERTSUBJECTS)
-				{
-					fprintf(qp, "%s%c%s%c", fnamebuf, 0, subject, 0);
-				}
+  		 	 	  fprintf(qp, "%s%c%s%c", fnamebuf, 0, subject, 0);
 			}
 			else
 			{
@@ -884,3 +861,51 @@ int findmax(char *subject, int *this)
 	return(maxparts);
 }
 
+/* 
+ * finds the filename on the subject line, fills it to fnamebuf,
+ * and sets the curpart and maxparts if found.
+ */
+int findfilename(regex_t *preq, char *subject, char *fnamebuf, int *curpart, int *maxparts) 
+{
+	regmatch_t pmatch[1];
+	char *tmp=NULL;
+    int laststart=0,lastend=0;
+
+	// Find out where the last matching filename extension on the line starts
+	while(regexec(preq, subject, 1, pmatch, 0)!=REG_NOMATCH)
+	{
+		laststart=pmatch[0].rm_so;
+		lastend=pmatch[0].rm_eo;
+		tmp=subject;
+		subject=&subject[lastend];
+	}
+	
+	// If extension was found, parse backwards to get the filename to fnamebuf
+	if(tmp)
+	{
+		int i,j;
+		
+		for(i=laststart;
+			i>=0 && tmp[i]!=' ' && tmp[i]!='\"' && tmp[i]!='/' && tmp[i]!='\\';
+			i--);
+			
+		i++;
+		//if(tmp[i]==' ') i++;
+			
+		j=0;
+		while(i<lastend)
+			fnamebuf[j++]=tmp[i++];
+		fnamebuf[j]=0;
+
+//		if(Verbose)
+//			printf("  FILE %s\n", fnamebuf);
+
+	    // finally, find out the maximum number of parts from the subject line 
+		*maxparts=findmax(&tmp[i], curpart);
+
+		return(RESULT_OK);
+    } else {
+		return(-1);
+	}
+}
+
